/**
* Copyright (C) 2003-2019, Foxit Software Inc..
* All Rights Reserved.
*
* http://www.foxitsoftware.com
*
* The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
* distribute any parts of Foxit PDF SDK to third party or public without permission unless an agreement
* is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
*
* @file fs_convert.h
* @brief Header file for PDF conversion related definitions and classes.
*/

#ifndef FS_CONVERT_H_
#define FS_CONVERT_H_

#include "common/fs_common.h"
#include "pdf/fs_pdfdoc.h"
#include "pdf/fs_pdfpage.h"

/**
* @brief Foxit namespace.
*
* @details All the definitions and classes of Foxit PDF SDK are defined within this namespace.
*/
namespace foxit {
/**
 * @brief Add-on namespace.
 */
namespace addon {
/**
 * @brief Conversion namespace.
 */
namespace conversion {
/** This class represents setting data used for converting HTML to PDF. */
class HTML2PDFSettingData  FS_FINAL : public Object{
 public:
  /**
   * @brief Enumeration for page mode used for converting HTML to PDF.
   *
   * @details Values of this enumeration should be used alone.
   */
  typedef enum _HTML2PDFPageMode {
    /** @brief Single page mode.*/
    e_PageModeSinglePage  = 0,
    /** @brief Multiple page mode. */
    e_PageModeMultiplePage = 1
  } HTML2PDFPageMode;
  
  /** @brief Constructor. */
  HTML2PDFSettingData()
        : page_width(0)
        , page_height(0)
        , is_to_page_scale(false)
        , page_margin(0.0f, 0.0f, 0.0f, 0.0f)
        , rotate_degrees(common::e_Rotation0)
        , is_convert_link(false)
        , is_generate_tag(false)
        , page_mode(e_PageModeSinglePage)
        {}

  /**
   * @brief Constructor, with parameters.
   *
   * @param[in] page_width        The page width used for the coverted PDF document.
   * @param[in] page_height       The page height used for the coverted PDF document. This would be ignored if page mode
   *                              @link HTML2PDFSettingData::e_PageModeSinglePage @endlink is used.
   * @param[in] is_to_page_scale  Used to decide whether to scale the coverted PDF page or not.
   * @param[in] page_margin       The page margin used for coverted PDF document.
   * @param[in] is_convert_link   Used to decide whether the web link should be coverted or not.
   * @param[in] rotate_degrees    The rotation degree used for the coverted PDF page.Please refer to values starting from @link common::e_Rotation0 @endlink and
   *                              this should be one of these values. 
   * @param[in] is_generate_tag   Used to decide whether the tag should be generated or not.
   * @param[in] page_mode         The page mode used for the converting.. Please refer to values starting from @link HTML2PDFSettingData::e_PageModeSinglePage @endlink and
   *                              this should be one of these values.
   *
   * @return None.
   */
  HTML2PDFSettingData(float page_width, float page_height, bool is_to_page_scale, RectF page_margin,
      bool is_convert_link, common::Rotation rotate_degrees, bool is_generate_tag, HTML2PDFPageMode page_mode) {
    this->page_width = page_width;
    this->page_height = page_height;
    this->is_to_page_scale = is_to_page_scale;
    this->page_margin = page_margin;
    this->is_convert_link = is_convert_link;
    this->rotate_degrees = rotate_degrees;
    this->is_generate_tag = is_generate_tag;
    this->page_mode = page_mode;
  }

  /**
   * @brief Assign operator.
   *
   * @param[in] data  Another HTML-to-pdf setting data object, whose value would be assigned to current object.
   *
   * @return Reference to current object itself.
   */
  HTML2PDFSettingData &operator = (const HTML2PDFSettingData& data) {
    page_width = data.page_width;
    page_height = data.page_height;
    is_to_page_scale = data.is_to_page_scale;
    page_margin = data.page_margin;
    is_convert_link = data.is_convert_link;
    rotate_degrees = data.rotate_degrees;
    is_generate_tag = data.is_generate_tag;
    page_mode = data.page_mode;
    return (*this);
  }

  /** @brief The page width used for the coverted PDF document. */
  float          page_width;

  /**
   * @brief The page height used for the coverted PDF document.
   *
   * @note The page height would be ignored if page mode @link HTML2PDFSettingData::e_PageModeSinglePage @endlink is used.
   */
  float          page_height;

  /** @brief Used to decide whether to scale the coverted PDF page or not. */
  bool           is_to_page_scale;

  /**
   * @brief The page margin used for coverted PDF document.
   *
   * @note This value does not represents a rectangle. It just defines the left margin, bottom margin,
   *       right margin and top margin of a PDF page.
   */
  RectF          page_margin;

  /** 
   * @brief The rotation degree used for the coverted PDF page.Please refer to values starting from @link common::e_Rotation0 @endlink and
   *        this should be one of these values. 
   */
  common::Rotation       rotate_degrees;

  /** @brief Used to decide whether the web link should be coverted or not. */
  bool           is_convert_link;

  /** @brief Used to decide whether the tag should be generated or not. */
  bool           is_generate_tag;

  /**
   * @brief The page mode used for the converting. Please refer to values starting from @link HTML2PDFSettingData::e_PageModeSinglePage @endlink and
   *        this should be one of these values.
   */
  HTML2PDFPageMode page_mode;
};

/**
 * Conversion add-on module can be used to do some conversion between PDF file and other file formats.
 * Before using "Conversion" module, user should first prepare font resource folder as described in "Foxit PDF SDK Developer Guide.pdf" document.
 * Before using any class or methods in this module, please ensure Foxit PDF SDK has been initialized successfully
 * by function @link common::Library::Initialize @endlink with a key including "Conversion" module.
 *
 * @see @link common::Library @endlink
 */
class Convert FS_FINAL : public Base {
 public:
   /**
    * @brief Convert a HTML format file or a url of a Web page to a PDF file.
    *
    * @details Currently, this function can only be used for Windows and Mac OS; for other platform, this function will
    *          return directly without doing anything.
    *
    * @param[in] src_html        A URL of Web page or a HTML format file path. This should not be an empty string.
    * @param[in] engine_path     The directory path of "HTML to PDF" engine. This should not be an empty string.
    * @param[in] cookies_path    Path of cookies file. If this is an empty string, that means no cookie file is used.
    * @param[in] setting_data    Setting data used for converting.
    * @param[in] saved_pdf_path  A path for saving the converted PDF file. This should not be an empty string.
    * @param[in] timeout         Seconds for timeout used for loading webpages. It should be set to a value greater than 15.
    *                            If this is less than 15, 15 will be used by default.
    *
    * @return None.
    */
   static void FromHTML(const wchar_t* src_html, const wchar_t* engine_path, const wchar_t* cookies_path, const HTML2PDFSettingData& setting_data, const wchar_t* saved_pdf_path, int32 timeout);

};
}
}
}
#endif
