/**
 * Copyright (C) 2003-2019, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 *
 * @file fs_connectedpdf.h
 * @brief Header file for ConnectedPDF related definitions and classes.
 */

#ifndef FS_CONNECTEDPDF_H_
#define FS_CONNECTEDPDF_H_

#include "common/fs_common.h"
#include "pdf/fs_pdfdoc.h"

/**
 * @brief Foxit namespace.
 *
 * @details All the definitions and classes of Foxit PDF SDK are defined within this namespace.
 */
namespace foxit {
/**
 * @brief Add-on namespace.
 */
namespace addon {
/**
 * Class @link ConnectedPDF @endlink offers following functions,
 * <ul>
 * <li>Convert a normal PDF document to a ConnectedPDF document.</li>
 * <li>Convert a normal PDF document or a ConnectedPDF document to a ConnectedPDF DRM document.</li>
 * <li>Decrypt and configure the ACL of ConnectedPDF DRM.</li>
 * </ul>
 */
class ConnectedPDF FS_FINAL : public Base {
 public:
   /**
    * @brief Enumeration for encryption mode.
    *
    * @details Values of this enumeration should be used alone.
    */
   typedef enum _EncryptType {
     /** @brief AES encryption algorithm.*/
     e_EncryptTypeAES = 1,
     /** @brief SM4 electronic codebook mode.*/
     e_EncryptTypeSM4CBC = 2
   } EncryptType;
   

#if (_FX_OS_ != _FX_IOS_ && !defined(_SWIG_IOS_) && !defined(_OC_) && !defined(_SWIG_MACOC_))
  /**
    * @brief Structure that represents the client information which is used to apply for a client ID
    *        from ConnectedPDF server.
    *
    * @details The client ID represents a ConnectedPDF client running on a device. The client should
    *          apply for a client ID once, then cache it and reuse it.
    */
  class ClientInfo : public Object {
   public:
    /** @brief The device ID that represents a device.*/
    String device_id;
    /** @brief The device name.*/
    String device_name;
    /** @brief The device model.*/
    String device_model;
    /** @brief The MAC address.*/
    String mac_address;
    /** @brief The operation system.*/
    String os;
    /** @brief The product name of a ConnectedPDF client.*/
    String product_name;
    /** @brief The vendor of the product.*/
    String product_vendor;
    /** @brief The version of the product.*/
    String product_version;
    /** @brief The language of the product.*/
    String product_language;
  };
#endif  // (_FX_OS_ != _FX_IOS_ && !defined(_SWIG_IOS_) && !defined(_OC_) && !defined(_SWIG_MACOC_))

  /**
   * @brief Get the ConnectedPDF document URI from the PDF path.
   *
   * @details Every ConnectedPDF document or ConnectedPDF DRM document has the document URI.
   *          It is the unique identification of the ConnectedPDF document or ConnectedPDF DRM document.
   *
   * @param[in] file_path  A path of an existing PDF file.
   *
   * @return The ConnectedPDF document URI. If the returned string is an empty string,
   *         that means the input document is not a valid ConnectedPDF document.
   *
   * @note If module "ConnectedPDF" is not defined in the license information which is used in function
   *       @link common::Library::Initialize @endlink, that means user has no right in using ConnectedPDF related functions and
   *       this function will throw exception @link foxit::e_ErrInvalidLicense @endlink.
   */
   static String GetDocURI(const char* file_path);

  /**
   * @brief Apply for a client id from ConnectedPDF server using the input client info.
   *
   * @details The client id represents a ConnectedPDF client running on a device. The client ID is used to
   *          create a @link ConnectedPDF @endlink object. It is required for converting normal PDF to ConnectedPDF.
   *          The client should apply for a client id once, then cache it and reuse it.
   *          This function may throw exception @link foxit::e_ErrCanNotConnectToServer @endlink.
   *
   * @param[in] endpoint     The input ConnectedPDF server host address. For example,
   *                         https://www-fz02.connectedpdf.com.
   * @param[in] client_info  The input client information used to apply for a client id from ConnectedPDF server.
   *
   * @return The client ID string. If the returned string is not an empty string, that means success.
   *
   * @note If module "ConnectedPDF" is not defined in the license information which is used in function
   *       @link common::Library::Initialize @endlink, that means user has no right in using ConnectedPDF related functions and
   *       this function will throw exception @link foxit::e_ErrInvalidLicense @endlink.
   */
  static String GetClientIDFromServer(const char* endpoint, ClientInfo* client_info);

  /**
   * @brief Get the user token from ConnectedPDF server.
   *
   * @details A user token should be gotten before accessing ConnectedPDF web service. The client can get the user
   *          token once, then cache it and reuse it. This function may throw exception
   *          @link foxit::e_ErrCanNotConnectToServer @endlink.
   *
   * @param[in] endpoint    The input ConnectedPDF server host address. For example, https://www-fz02.connectedpdf.com.
   * @param[in] user_email  The input user email.
   * @param[in] password    The input user password.
   *
   * @return The user token string. If the returned string is not an empty string, that means success.
   *
   * @note If module "ConnectedPDF" is not defined in the license information which is used in function
   *       @link common::Library::Initialize @endlink, that means user has no right in using ConnectedPDF related functions and
   *       this function will throw exception @link foxit::e_ErrInvalidLicense @endlink.
   */
  static String GetUserTokenFromServer(const char* endpoint, const String& user_email, const String& password);

  /**
   * @brief Constructor, with parameters and from an existing PDF file path.
   *
   * @details A @link ConnectedPDF @endlink object can be constructed to load a normal PDF document, or a ConnectedPDF document,
   *          or a ConnectedPDF DRM document. If current document is a normal PDF document, it can be converted to
   *          a ConnectedPDF document or a ConnectedPDF DRM document. If current document is a ConnectedPDF document,
   *          it can be convert to a ConnectedPDF DRM document.
   *
   * @param[in] clientid     A client ID whcih is created by function @link ConnectedPDF::GetClientIDFromServer @endlink.
   * @param[in] user_token   A user token is created by function @link ConnectedPDF::GetUserTokenFromServer @endlink.
   * @param[in] content_key  If the client has cached the content key, just input the cached key to
   *                         decrypt ConnectedPDF DRM document. Otherwise, set <i>content_key</i> with an empty string,
   *                         and @link ConnectedPDF @endlink object will apply for the content key from server.
   * @param[in] type         Encryption algorithm. Please refer to values starting from @link ConnectedPDF::e_EncryptTypeAES @endlink
   *                         and this should be one of these values.
   * @param[in] file_path    A path of an existing PDF file.
   *
   * @note If module "ConnectedPDF" is not defined in the license information which is used in function
   *       @link common::Library::Initialize @endlink, that means user has no right in using ConnectedPDF related functions and
   *       this constructor will throw exception @link foxit::e_ErrInvalidLicense @endlink.
   */
  ConnectedPDF(const char* clientid, const char* user_token, const char* content_key, addon::ConnectedPDF::EncryptType type, const char* file_path);

  /**
   * @brief Constructor, with parameters and from an existing PDF file path.
   *
   * @details A @link ConnectedPDF @endlink object can be constructed to load a normal PDF document, or a ConnectedPDF document,
   *          or a ConnectedPDF DRM document. If current document is a normal PDF document, it can be converted to
   *          a ConnectedPDF document or a ConnectedPDF DRM document. If current document is a ConnectedPDF document,
   *          it can be convert to a ConnectedPDF DRM document.
   *
   * @param[in] clientid     A client ID whcih is created by function @link ConnectedPDF::GetClientIDFromServer @endlink.
   * @param[in] user_token   A user token is created by function @link ConnectedPDF::GetUserTokenFromServer @endlink.
   * @param[in] content_key  If the client has cached the content key, just input the cached key to
   *                         decrypt ConnectedPDF DRM document. Otherwise, set <i>content_key</i> with an empty string,
   *                         and @link ConnectedPDF @endlink object will apply for the content key from server.
   * @param[in] type         Encryption algorithm. Please refer to values starting from @link ConnectedPDF::e_EncryptTypeAES @endlink
   *                         and this should be one of these values.
   * @param[in] file_path    A path of an existing PDF file.
   *
   * @note If module "ConnectedPDF" is not defined in the license information which is used in function
   *       @link common::Library::Initialize @endlink, that means user has no right in using ConnectedPDF related functions and
   *       this constructor will throw exception @link foxit::e_ErrInvalidLicense @endlink.
   */
  ConnectedPDF(const char* clientid, const char* user_token, const char* content_key,addon::ConnectedPDF::EncryptType type, const wchar_t* file_path);

  /**
   * @brief Constructor, with parameters and a @link common::file::ReaderCallback @endlink object.
   *
   * @details A @link ConnectedPDF @endlink object can be constructed to load a normal PDF document, or a ConnectedPDF document,
   *          or a ConnectedPDF DRM document. If current document is a normal PDF document, it can be converted to
   *          a ConnectedPDF document or a ConnectedPDF DRM document. If current document is a ConnectedPDF document,
   *          it can be convert to a ConnectedPDF DRM document.
   *
   * @param[in] clientid     A client ID whcih is created by function @link ConnectedPDF::GetClientIDFromServer @endlink.
   * @param[in] user_token   A user token is created by function @link ConnectedPDF::GetUserTokenFromServer @endlink.
   * @param[in] content_key  If the client has cached the content key, just input the cached key to
   *                         decrypt ConnectedPDF DRM document. Otherwise, set <i>content_key</i> with an empty string,
   *                         and @link ConnectedPDF @endlink object will apply for the content key from server.
   * @param[in] type         Encryption algorithm. Please refer to values starting from @link ConnectedPDF::e_EncryptTypeAES @endlink
   *                         and this should be one of these values.
   * @param[in] file_read    A @link common::file::ReaderCallback @endlink object which is implemented by user to
   *                         load a PDF document. It should not be <b>NULL</b>.
   *
   * @note If module "ConnectedPDF" is not defined in the license information which is used in function
   *       @link common::Library::Initialize @endlink, that means user has no right in using ConnectedPDF related functions and
   *       this constructor will throw exception @link foxit::e_ErrInvalidLicense @endlink.
   */
  ConnectedPDF(const char* clientid, const char* user_token, const char* content_key,addon::ConnectedPDF::EncryptType type, foxit::common::file::ReaderCallback* file_read);

  /**
   * @brief Constructor, with parameters and a PDF document object.
   *
   * @details The constructed @link ConnectedPDF @endlink object can be converted to a ConnectedPDF document
   *          or a ConnectedPDF DRM document.
   *
   * @param[in] clientid     A client ID whcih is created by function @link ConnectedPDF::GetClientIDFromServer @endlink.
   * @param[in] user_token   A user token is created by function @link ConnectedPDF::GetUserTokenFromServer @endlink.
   * @param[in] content_key  If the client has cached the content key, just input the cached key to
   *                         decrypt ConnectedPDF DRM document. Otherwise, set <i>content_key</i> as <b>NULL</b>,
   *                         and @link ConnectedPDF @endlink object will apply for the content key from server.
   * @param[in] type         Encryption algorithm. Please refer to values starting from @link ConnectedPDF::e_EncryptTypeAES @endlink
   *                         and this should be one of these values.
   * @param[in] document     A valid PDF document object which represents a normal PDF document.
   *
   * @note If module "ConnectedPDF" is not defined in the license information which is used in function
   *       @link common::Library::Initialize @endlink, that means user has no right in using ConnectedPDF related functions and
   *       this constructor will throw exception @link foxit::e_ErrInvalidLicense @endlink.
   */
    ConnectedPDF(const foxit::String& clientid, const foxit::String& user_token, const foxit::String& content_key, addon::ConnectedPDF::EncryptType type,const foxit::pdf::PDFDoc& document);

  /**
   * @brief Constructor, with another @link ConnectedPDF @endlink object.
   *
   * @param[in] connected_pdf  Another @link ConnectedPDF @endlink object
   */
  ConnectedPDF(const ConnectedPDF& connected_pdf);
  /** @brief Destructor function. */
  ~ConnectedPDF();

  /**
   * @brief Assign operator.
   *
   * @param[in] other  Another @link ConnectedPDF @endlink object, whose value would be assigned to current object.
   *
   * @return Reference to current object itself.
   */
  ConnectedPDF& operator = (const ConnectedPDF& other);
  /**
   * @brief Equal operator.
   *
   * @param[in] other  Another @link ConnectedPDF @endlink object. This function will check if current object is equal to this one.
   *
   * @return <b>true</b> means equal, while <b>false</b> means not equal.
   */
  bool operator == (const ConnectedPDF& other) const ;
  /** @brief Not equal operator. */
  /**
   * @brief Not equal operator.
   *
   * @param[in] other  Another @link ConnectedPDF @endlink object. This function will check if current object is not equal to this one.
   *
   * @return <b>true</b> means not equal, while <b>false</b> means equal.
   */
  bool operator != (const ConnectedPDF& other) const ;

  /**
   * @brief Check whether current object is empty or not.
   *
   * @details When the current object is empty, that means current object is useless.
   *
   * @return <b>true</b> means current object is empty, while <b>false</b> means not.
   */
  bool IsEmpty() const;

  /**
   * @brief Check whether the loaded document is a ConnectedPDF document or not.
   *
   * @return <b>true</b> means the loaded document is a ConnectedPDF document,
   *         while <b>false</b> means the loaded document is not a ConnectedPDF document.
   */
  bool IsCPDF();

  /**
   * @brief Check whether the loaded document is a ConnectedPDF DRM document or not.
   *
   * @return <b>true</b> means the loaded document is a ConnectedPDF DRM document,
   *         while <b>false</b> means the loaded document is not a ConnectedPDF DRM document.
   */
  bool IsCDRM();

  /**
   * @brief Check whether current user is the owner.
   *
   * @return <b>true</b> means current user is the owner, while <b>false</b> means current user is not the owner.
   */
  bool IsOwner();

  /**
   * @brief Get the associated PDF document object.
   *
   * @details The returned PDF document object can be edited. If the loaded document is a ConnectedPDF DRM document,
   *          user should save it by function @link ConnectedPDF::SaveAs @endlink; otherwise, the saved document
   *          will be corrupted.
   *
   * @return The associated PDF document object.
   */
  foxit::pdf::PDFDoc GetDocument();

  /**
   * @brief Set the user token to switch it.
   *
   * @param[in] user_token  The input user token.
   *
   * @return None.
   */
  void SetUserToken(const String& user_token);

  /**
   * @brief Get the user token.
   *
   * @return The user token.
   */
  String GetUserToken();

  /**
   * @brief Get the content key if the loaded document is a ConnectedPDF DRM document.
   *
   * @return The content key of ConnectedPDF DRM document.
   */
  String GetContentKey();

  /**
   * @brief Convert the loaded document to a ConnectedPDF document.
   *
   * @details When the loaded document is converted to a ConnectedPDF document, please use function
   *          @link ConnectedPDF::SaveAs @endlink to save it.
   *
   * @param[in] endpoint   The input ConnectedPDF server host address. For example, https://www-fz02.connectedpdf.com.
   * @param[in] file_name  The file name used by ConnectedPDF server. If this is an empty string,
   *                       "Unknown.pdf" will be used as the file name by default
   *
   * @return @link foxit::e_ErrSuccess @endlink means success. <br>
   *         @link foxit::e_ErrIsAlreadyCPDF @endlink means the loaded document has been a ConnectedPDF document.<br>
   *         For other error code value, please refer to values starting from @link foxit::e_ErrSuccess @endlink for more details.
   */
  foxit::ErrorCode ConvertToCPDF(const char* endpoint, const char* file_name);

  /**
   * @brief Convert the loaded document to a ConnectedPDF document.
   *
   * @details When the loaded document is converted to a ConnectedPDF document, please use function
   *          @link ConnectedPDF::SaveAs @endlink to save it.
   *
   * @param[in] endpoint   The input ConnectedPDF server host address. For example, https://www-fz02.connectedpdf.com.
   * @param[in] file_name  The file name used by ConnectedPDF server. If this is an empty string,
   *                       "Unknown.pdf" will be used as the file name by default
   *
   * @return @link foxit::e_ErrSuccess @endlink means success. <br>
   *         @link foxit::e_ErrIsAlreadyCPDF @endlink means the loaded document has been a ConnectedPDF document.<br>
   *         For other error code value, please refer to values starting from @link foxit::e_ErrSuccess @endlink for more details.
   */
  foxit::ErrorCode ConvertToCPDF(const char* endpoint, const wchar_t* file_name);

  /**
   * @brief Convert the loaded document to a ConnectedPDF DRM document.
   *
   * @details When the loaded document is converted to a ConnectedPDF DRM document, please use function
   *          @link ConnectedPDF::SaveAs @endlink to save it.
   *
   * @param[in] endpoint      The input ConnectedPDF server host address. For example,
   *                          https://www-fz02.connectedpdf.com.
   * @param[in] wrapper_path  A path of an existing PDF file.
   * @param[in] file_name     The file name used by ConnectedPDF server. If this is an empty string,
   *                          "Unknown.pdf" will be used as the file name by default
   *
   * @return @link foxit::e_ErrSuccess @endlink means success. <br>
   *         @link foxit::e_ErrIsAlreadyCDRM @endlink means the loaded document has been a ConnectedPDF DRM document.<br>
   *         For other error code value, please refer to values starting from @link foxit::e_ErrSuccess @endlink for more details.
   */
  foxit::ErrorCode ConvertToCDRM(const char* endpoint, const char* wrapper_path, const char* file_name);

  /**
   * @brief Convert the loaded document to a ConnectedPDF DRM document.
   *
   * @details When the loaded document is converted to a ConnectedPDF DRM document, please use function
   *          @link ConnectedPDF::SaveAs @endlink to save it.
   *
   * @param[in] endpoint      The input ConnectedPDF server host address. For example,
   *                          https://www-fz02.connectedpdf.com.
   * @param[in] wrapper_path  A path of an existing PDF file.
   * @param[in] file_name     The file name used by ConnectedPDF server. If this is an empty string,
   *                          "Unknown.pdf" will be used as the file name by default
   *
   * @return @link foxit::e_ErrSuccess @endlink means success. <br>
   *         @link foxit::e_ErrIsAlreadyCDRM @endlink means the loaded document has been a ConnectedPDF DRM document.<br>
   *         For other error code value, please refer to values starting from @link foxit::e_ErrSuccess @endlink for more details.
   */
  foxit::ErrorCode ConvertToCDRM(const char* endpoint, const wchar_t* wrapper_path, const wchar_t* file_name);

  /**
   * @brief Convert the loaded document to a ConnectedPDF DRM document.
   *
   * @details When the loaded document is converted to a ConnectedPDF DRM document, please use function
   *          @link ConnectedPDF::SaveAs @endlink to save it.
   *
   * @param[in] endpoint           The input ConnectedPDF server host address. For example,
   *                               https://www-fz02.connectedpdf.com.
   * @param[in] wrapper_file_read  A @link common::file::ReaderCallback @endlink object which is implemented by user to save a PDF document.
   * @param[in] file_name          The file name used by ConnectedPDF server. If this is an empty string,
   *                               "Unknown.pdf" will be used as the file name by default
   *
   * @return @link foxit::e_ErrSuccess @endlink means success. <br>
   *         @link foxit::e_ErrIsAlreadyCDRM @endlink means the loaded document has been a ConnectedPDF DRM document.<br>
   *         For other error code value, please refer to values starting from @link foxit::e_ErrSuccess @endlink for more details.
   */
  foxit::ErrorCode ConvertToCDRM(const char* endpoint, foxit::common::file::ReaderCallback* wrapper_file_read, const char* file_name);

  /**
   * @brief Convert the loaded document to a ConnectedPDF DRM document.
   *
   * @details When the loaded document is converted to a ConnectedPDF DRM document, please use function
   *          @link ConnectedPDF::SaveAs @endlink to save it.
   *
   * @param[in] endpoint           The input ConnectedPDF server host address. For example,
   *                               https://www-fz02.connectedpdf.com.
   * @param[in] wrapper_file_read  A @link common::file::ReaderCallback @endlink object which is implemented by user to save a PDF document.
   * @param[in] file_name          The file name used by ConnectedPDF server. If this is an empty string,
   *                               "Unknown.pdf" will be used as the file name by default
   *
   * @return @link foxit::e_ErrSuccess @endlink means success. <br>
   *         @link foxit::e_ErrIsAlreadyCDRM @endlink means the loaded document has been a ConnectedPDF DRM document.<br>
   *         For other error code value, please refer to values starting from @link foxit::e_ErrSuccess @endlink for more details.
   */
  foxit::ErrorCode ConvertToCDRM(const char* endpoint, foxit::common::file::ReaderCallback* wrapper_file_read, const wchar_t* file_name);

  /**
   * @brief Remove the ConnectedPDF DRM protection of a ConnectedPDF document.
   *
   * @details Only the owner has the permission to remove the ConnectedPDF DRM protection.
   *          When the ConnectedPDF DRM protection is removed, function @link ConnectedPDF::SaveAs @endlink can be used to
   *          save it.
   *
   * @return @link foxit::e_ErrSuccess @endlink means success. <br>
   *         @link foxit::e_ErrNotCDRM @endlink means the loaded document is not a ConnectedPDF DRM document.<br>
   *         For other error code value, please refer to values starting from @link foxit::e_ErrSuccess @endlink for more details.
   */
  foxit::ErrorCode RemoveCDRM();

  /**
   * @brief Set ACL of a ConnectedPDF DRM document.
   *
   * @details Only the owner has the permission to set ACL.
   *          This function will revoke all the ACL of the ConnectedPDF DRM document first, and then apply the new ACL.
   *
   * @param[in] acl  The input ACL data. The data is a JSON string. "userList" is an array of all the user's ACL.
   *                 Every element of "userList" is one user's ACL. The format is described as follows,
   <pre>
   {
     "userList":
     [
       {
         The user email.
         "userEmail": "fz02_50@qq.com",<br>
         The grout ID. Cannot support now. Just leave it empty.
         "groupId": "",<br>
         Whether the ACL is for a group or not. Cannot support now. Just leave it 0.
         "isGroup": "0",<br>
         Whether the ACL is for everyone or not.
         "everyone": "0",<br>
         Whether the user has full control permission.
         "fullControl": "0",<br>
         The PDF standard permission array. The value is hex. The values can be 4,8,10,20,100,200,400,800.
         Please refer to values starting from @link pdf::PDFDoc::e_PermPrint @endlink for more information.
         "permission": [4],<br>
         Whether the user can open the document offline. 0=Never 1=By Days 2=Always.
         "offline": "0",<br>
         The permitted offline days.
         "offlineDays": 0,<br>
         Whether set the online expiration date. 0=Never 1=By Days 2=By Date.
         "online": "0",<br>
         Expiration date. For example, "2017-01-01 01:59:59".
         "onlineDate": "",<br>
         Expiration days.
         "onlineDays": "0",<br>
         The permitted device count to open the document.
         "deviceLimit": "0",<br>
         Control the specified pages to be hidden.
         "accessPages":
         {
           The content cover over the hidden pages.
           "wrapperContent": "",<br>
           The picture cover over the hidden pages. Cannot support now. Just leave it empty.
           "wrapperPic": "",<br>
           The page indexes to be hidden. For example, [ 1, [2,10],12 ].
           "Pages": [0]
         }
       }
     ]
   }
   </pre>
   *
   * @return @link foxit::e_ErrSuccess @endlink means success. <br>
   *         @link foxit::e_ErrInvalidACL @endlink means the input ACL data is invalid.<br>
   *         @link foxit::e_ErrNotCDRM @endlink means the loaded document is not a ConnectedPDF DRM document.<br>
   *         For other error code value, please refer to values starting from @link foxit::e_ErrSuccess @endlink for more details.
   */
  foxit::ErrorCode SetACL(const String& acl);

  /**
   * @brief Get ACL of a ConnectedPDF DRM document.
   *
   * @details The owner can get all the user's ACL; otherwise, the current user can get its own ACL.
   *          If the loaded document is not a ConnectedPDF DRM document, this function will throw exception
   *          @link foxit::e_ErrNotCDRM @endlink.
   *
   * @return The ACL data string. The data is a JSON string. The JSON string is an array of all the user's ACL.
   *         Every element of the array is one user's ACL. The format is described as follows,
   <pre>
   [
     {
       Control the specified pages to be hidden.
       "accessPages" :
       {
         The page indexes to be hidden. For example, [ 1, [2,10],12 ].
         "Pages" : [],
         The content cover over the hidden pages.
         "wrapperContent" : null,
         The picture cover over the hidden pages. Cannot support now. Just leave it empty.
         "wrapperPic" : null
       },
       The PDF standard permission array. The value is hex. The values can be 4,8,10,20,100,200,400,800.
       Please refer to values starting from @link pdf::PDFDoc::e_PermPrint @endlink for more information.
       "decodePermission" : [ 8, 10 ],
       Limit device to open the document.
       "deviceLimit" :
       {
         The devices has opened the document.
         "appList" : [],
         No more device can open the document.
         "isLimit" : 0,
         The permitted device count to open the document.
         "limitNum" : 0
       },
       Whether the ACL is for everyone or not.
       "everyone" : 0,
       Whether the user has full control permission.
       "fullControl" : 0,
       The grout ID.
       "groupID" : "",
       The grout name.
       "groupName" : "",
       Whether the ACL is for a group or not.
       "isGroup" : 0,
       Whether the ACL has been revoked.
       "isRevoke" : 0,
       Whether the user can open the document offline. 0=Never 1=By Days 2=Always.
       "offline" : 0,
       The permitted offline days.
       "offlineDuration" : 0,
       Whether set the online expiration date. 0=Never 1=By Days 2=By Date.
       "online" : 0,
       Expiration date. For example, "2017-01-01 01:59:59".
       "onlineDuration" : "",
       Expiration days.
       "onlineDurationDays" : 0,
       The PDF standard permission. The value is decimal. It's the combination of the hex values 4,8,10,20,100,200,400,800.
       Please refer to values starting from @link pdf::PDFDoc::e_PermPrint @endlink for more information.
       "permission" : 312,
       The user info.
       "userAttribute" :
       {
         The domain the user belongs to.
         "Domain" : "",
         The user ID.
         "cUserID" : "58197659c2dcf1c7143febce",
         The user email.
         "userEmail" : "65908769@qq.com"
       }
     }
   ]
   </pre>
   */
  String GetACL();
  /**
   * @brief Get the ConnectedPDF document URI from the loaded document.
   *
   * @return The ConnectedPDF document URI. If the returned string is an empty string, that means
   *         the loaded document is not a valid ConnectedPDF document.
   */
  String GetDocURI();
  /**
   * @brief Get the ConnectedPDF document version ID from the loaded document.
   *
   * @return The ConnectedPDF document version ID. If the returned string is an empty string, that means
   *         the loaded document is not a valid ConnectedPDF document.
   */
  String GetVersionID();  
  /**
   * @brief Refresh the ConnectedPDF document version ID from the loaded document.
   *
   * @return The ConnectedPDF document new version ID. If the returned string is an empty string, that means
   *         the loaded document is not a valid ConnectedPDF document.
   */
  String RefreshVersionID();
  /**
   * @brief Get the ConnectedPDF document endpoint from the loaded document.
   *
   * @return The ConnectedPDF document endpoint. If the returned string is an empty string, that means
   *         the loaded document is not a valid ConnectedPDF document.
   */
  String GetEndpoint();
  /**
   * @brief Get the ConnectedPDF document owner email.
   *
   * @return The ConnectedPDF document owner email.
   */
  String GetOwnerEmail();
  /**
   * @brief Get the ConnectedPDF document owner name.
   *
   * @return The ConnectedPDF document owner name.
   */
  String GetOwnerName();
  /**
   * @brief Save the current ConnectedPDF document as another PDF file.
   *
   * @details If the loaded document is a ConnectedPDF DRM document, the document can be saved as only
   *          by this funcction; otherwise the saved document will be corrupted.
   *
   * @param[in] file_path  A path for the new saved PDF file. It should not be an empty string.
   *
   * @return <b>true</b> means the saving is successfully finished, while <b>false</b> means failure.
   */
  bool SaveAs(const char* file_path);
  /**
   * @brief Save the current ConnectedPDF document as another PDF file.
   *
   * @details If the loaded document is a ConnectedPDF DRM document, the document can be saved as only
   *          by this funcction; otherwise the saved document will be corrupted.
   *
   * @param[in] file_write  A @link common::file::WriterCallback @endlink object which is implemented by user to do the saving.
   *
   * @return <b>true</b> means the saving is successfully finished, while <b>false</b> means failure.
   */
  bool SaveAs(foxit::common::file::WriterCallback* file_write);

  // User is strongly recommended NOT to use this method; otherwise unknown situation may occur.
  explicit ConnectedPDF(FS_HANDLE handle =  NULL);
};

}  // namespace addon
}  // namespace foxit
#endif  // FS_PDFDOC_H_

